@@ -25,7 +25,7 @@
 #
 # HUBzero is a registered trademark of The Regents of the University of California.
 #
-__version__ = '3.7.2'
+__version__ = '3.8.1'
 
 import sys
 import os
@@ -43,6 +43,7 @@ import math
 import copy
 import shutil
 import logging
+import json
 from errno import EPIPE
 
 from hubzero.submit.LogMessage            import getLogJobIdMessage as getLogMessage, logSetJobId
@@ -208,9 +209,9 @@ class JobHarvester:
 
 
    def configure(self):
-      sectionPattern  = re.compile('(\s*\[)([^\s]*)(]\s*)')
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      sectionPattern  = re.compile(r'(\s*\[)([^\s]*)(]\s*)')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       inDistributorSection = False
 
       configured = False
@@ -250,23 +251,30 @@ class JobHarvester:
                               self.configData[key] = int(value)
                            elif isinstance(self.configData[key],dict):
                               try:
-                                 sampleKey   = self.configData[key].keys()[0]
+                                 sampleKey   = list(self.configData[key].keys())[0]
                                  sampleValue = self.configData[key][sampleKey]
                               except:
-                                 sampleKey   = "key"
-                                 sampleValue = "value"
-                              self.configData[key] = {} 
-                              for e in value.split(','):
-                                 dictKey,dictValue = e.split(':')
-                                 if isinstance(sampleKey,int):
-                                    dictKey = int(dictKey)
-                                 if   isinstance(sampleValue,int):
-                                    dictValue = int(dictValue)
-                                 elif isinstance(sampleValue,float):
-                                    dictValue = float(dictValue)
-                                 elif isinstance(sampleValue,bool):
-                                    dictValue = bool(dictValue.lower() == 'true')
-                                 self.configData[key][dictKey] = dictValue
+                                 try:
+                                    self.configData[key] = json.loads(value)
+                                 except:
+                                    self.configData[key] = {}
+                                    sampleKey   = "key"
+                                    sampleValue = "value"
+                              else:
+                                 self.configData[key] = {}
+               
+                              if not self.configData[key]:
+                                 for e in value.split(','):
+                                    dictKey,dictValue = e.split(':')
+                                    if isinstance(sampleKey,int):
+                                       dictKey = int(dictKey)
+                                    if   isinstance(sampleValue,int):
+                                       dictValue = int(dictValue)
+                                    elif isinstance(sampleValue,float):
+                                       dictValue = float(dictValue)
+                                    elif isinstance(sampleValue,bool):
+                                       dictValue = bool(dictValue.lower() == 'true')
+                                    self.configData[key][dictKey] = dictValue
                            else:
                               self.configData[key] = value
                         else:
@@ -361,10 +369,13 @@ class JobHarvester:
 
 
    def reportExitCondition(self):
+      runExitCode = 2
+      redundantJobKillStatus = 1 << 7 | signal.SIGUSR1
 #     self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(len(self.jobs)): %d" % (len(self.jobs))))
       if len(self.jobs) == 0:
-         jobIndices = self.jobStatistics.keys()
+         jobIndices = list(self.jobStatistics.keys())
          maximumJobIndex = max(jobIndices)
+#        self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(jobIndices): %s" % (jobIndices)))
 #        self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(maximumJobIndex): %d" % (maximumJobIndex)))
 
          try:
@@ -388,23 +399,28 @@ class JobHarvester:
             self.logger.log(logging.INFO,getLogMessage(message))
             message += "\n"
             os.write(3,message.encode('utf-8'))
+            runExitCode = exitCode
          except:
+            self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
             pass
       else:
 #        if self.successfulInstance == None:
-#           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(successfulInstance): None"))
+#           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(runType,successfulInstance): %s None" % \
+#                                                                                                   (self.runType)))
 #        else:
-#           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(successfulInstance): %d" % (self.successfulInstance)))
+#           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(runType,successfulInstance): %s %d" % \
+#                                                                           (self.runType,self.successfulInstance)))
          instanceIndex = 0
-         instances = self.jobs.keys()
+         instances = list(self.jobs.keys())
          instances.sort()
          for instance in instances:
 #           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(instance): %d" % (instance)))
             if (self.successfulInstance == None) or \
                (self.successfulInstance != None and (instance != self.successfulInstance)):
-               jobIndices = self.jobs[instance].jobStatistics.keys()
+               jobIndices = list(self.jobs[instance].jobStatistics.keys())
                jobIndices.sort()
                maximumJobIndex = max(jobIndices)
+#              self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(jobIndices): %s" % (jobIndices)))
 #              self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(maximumJobIndex): %d" % (maximumJobIndex)))
 
                try:
@@ -427,7 +443,8 @@ class JobHarvester:
 
                         instanceIndex += 1
                         message = "venue=%d:%s:%s:%s status=%d cputime=%f realtime=%f waittime=%f ncpus=%d" % \
-                              (instanceIndex,mechanism,remoteId,remoteLocation,exitCode,cpuTime,elapsedRunTime,waitTime,nCpuUsed)
+                                                            (instance,mechanism,remoteId,remoteLocation, \
+                                                             exitCode,cpuTime,elapsedRunTime,waitTime,nCpuUsed)
                         if event:
                            message += " event=%s" % (event)
                         if generated:
@@ -435,14 +452,46 @@ class JobHarvester:
                         self.logger.log(logging.INFO,getLogMessage(message))
                         message += "\n"
                         os.write(3,message.encode('utf-8'))
+                        if   self.runType == 'redundant':
+                           if exitCode != redundantJobKillStatus:
+                              if runExitCode == 2:
+                                 runExitCode = exitCode
+                        elif self.runType == 'pegasus':
+                           if instance == 0:
+                              if exitCode != 0:
+                                 runExitCode = exitCode
+                           else:
+                              if   runExitCode == 2:
+                                 runExitCode = exitCode
+                              elif runExitCode == 0:
+                                 runExitCode = exitCode
+                        elif self.runType == 'boinc':
+                           if instance == 0:
+                              if exitCode != 0:
+                                 runExitCode = exitCode
+                           else:
+                              if   runExitCode == 2:
+                                 runExitCode = exitCode
+                              elif runExitCode == 0:
+                                 runExitCode = exitCode
+                        elif self.runType == 'sweep':
+                           if   runExitCode == 2:
+                              runExitCode = exitCode
+                           elif runExitCode == 0:
+                              runExitCode = exitCode
                except:
+                  self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
                   pass
 
          if self.successfulInstance != None:
             instance = self.successfulInstance
-            jobIndices = self.jobs[instance].jobStatistics.keys()
+            jobIndices = list(self.jobs[instance].jobStatistics.keys())
             jobIndices.sort()
             maximumJobIndex = max(jobIndices)
+            waitForJobInfo = self.jobs[instance].getWaitForJobInfo()
+            executionMode = waitForJobInfo['executionMode']
+#           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(jobIndices): %s" % (jobIndices)))
+#           self.logger.log(logging.DEBUG,getLogMessage("reportExitCondition(maximumJobIndex): %d" % (maximumJobIndex)))
 
             try:
                for jobIndex in jobIndices:
@@ -469,23 +518,44 @@ class JobHarvester:
                      if instance == 0 and len(jobIndices) == 1 and \
                         (self.runType == 'pegasus' or self.runType == 'boinc'):
                         message = "venue=0:%s:%s:%s status=%d cputime=%f realtime=%f waittime=%f ncpus=%d" % \
-                                                            (              mechanism,remoteId,remoteLocation, \
+                                                          (         mechanism,remoteId,remoteLocation, \
                                                            exitCode,cpuTime,elapsedRunTime,waitTime,nCpuUsed)
                      else:
                         instanceIndex += 1
                         message = "venue=%d:%s:%s:%s status=%d cputime=%f realtime=%f waittime=%f ncpus=%d" % \
-                                                            (instanceIndex,mechanism,remoteId,remoteLocation, \
+                                                          (instance,mechanism,remoteId,remoteLocation, \
                                                            exitCode,cpuTime,elapsedRunTime,waitTime,nCpuUsed)
                      if event:
                         message += " event=%s" % (event)
                      if generated:
                         message += " generated=%s" % (generated)
+                     message += " executionMode=%s" % (executionMode)
                      self.logger.log(logging.INFO,getLogMessage(message))
                      message += "\n"
                      os.write(3,message.encode('utf-8'))
+                     if   self.runType == 'redundant':
+                        runExitCode = exitCode
+                     elif self.runType == 'pegasus':
+                        if instance == 0:
+                           if exitCode != 0:
+                              runExitCode = exitCode
+                     elif self.runType == 'boinc':
+                        if instance == 0:
+                           if exitCode != 0:
+                              runExitCode = exitCode
+                     elif self.runType == 'sweep':
+                        pass
             except:
+               self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
                pass
 
+      message = "runStatus=%d" % (runExitCode)
+      self.logger.log(logging.INFO,getLogMessage(message))
+      message += "\n"
+      os.write(3,message.encode('utf-8'))
+
+      return(runExitCode)
+
 
 # SIGTERM is sent by Rappture Abort
 # SIGHUP is sent by submit
@@ -632,8 +702,8 @@ class JobHarvester:
                            }
 
       informationLoaded = False
-      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
-      commentPattern  = re.compile('\s*#.*')
+      keyValuePattern = re.compile(r'( *)(\w*)( *= *)(.*[^\s$])( *)')
+      commentPattern  = re.compile(r'\s*#.*')
       try:
          fpHarvestInformation = open(harvestInformationPath,'r')
          try:
