@@ -38,7 +38,8 @@ from hubzero.submit.LogMessage             import getLogIDMessage as getLogMessa
 from hubzero.submit.JobStatistic           import JobStatistic
 from hubzero.submit.JobOutput              import JobOutput
 from hubzero.submit.VenueMechanismCore     import VenueMechanismCore
-from hubzero.submit.RemoteBatchTAPIS       import RemoteBatchTAPIS
+from hubzero.submit.RemoteBatchTAPIS2      import RemoteBatchTAPIS2
+from hubzero.submit.RemoteBatchTAPIS3      import RemoteBatchTAPIS3
 from hubzero.submit.RemoteBatchBOINC       import RemoteBatchBOINC
 from hubzero.submit.RemoteBatchCONDOR      import RemoteBatchCONDOR
 from hubzero.submit.RemoteBatchFACTORY     import RemoteBatchFACTORY
@@ -84,7 +85,8 @@ class VenueMechanismLocal(VenueMechanismCore):
                 isMultiCoreRequest,
                 disableJobMonitoring,
                 siteInfo,
-                tapisSiteInfo,
+                tapis2SiteInfo,
+                tapis3SiteInfo,
                 fileMoverInfo,
                 toolFilesInfo,
                 dockerImageInfo,
@@ -103,7 +105,7 @@ class VenueMechanismLocal(VenueMechanismCore):
                 pegasusHome):
       VenueMechanismCore.__init__(self,session,hubUserName,hubUserId,submitterClass,distributorPid,
                                        timeHistoryLogs,disableJobMonitoring,
-                                       siteInfo,tapisSiteInfo,fileMoverInfo,
+                                       siteInfo,tapis2SiteInfo,tapis3SiteInfo,fileMoverInfo,
                                        toolFilesInfo,dockerImageInfo,submissionScriptsInfo,managerInfo,
                                        remoteMonitors,isMultiCoreRequest,nCpus,nNodes,ppn,event)
 
@@ -118,7 +120,7 @@ class VenueMechanismLocal(VenueMechanismCore):
       else:
          self.preStageJobInput              = ""
          self.copyJobInput                  = ""
-      remoteScriptsDirectory = siteInfo['remoteBinDirectory'].replace('$','\$')
+      remoteScriptsDirectory = siteInfo['remoteBinDirectory'].replace('$',r'\$')
       self.receiveInput                     = os.path.join(remoteScriptsDirectory,batchCommands['receiveInput'])
       self.submitBatchJob                   = os.path.join(remoteScriptsDirectory,batchCommands['submitBatchJob'])
       self.postProcessJob                   = os.path.join(remoteScriptsDirectory,batchCommands['postProcessJob'])
@@ -162,7 +164,7 @@ class VenueMechanismLocal(VenueMechanismCore):
          self.arguments                     = ""
          for argument in arguments:
             if ' ' in argument:
-               self.arguments              += "%s " % (argument.replace('\\ ',' ').replace(' ','\ '))
+               self.arguments              += "%s " % (argument.replace('\\ ',' ').replace(' ',r'\ '))
             else:
                self.arguments              += "%s " % (argument)
       else:
@@ -511,7 +513,7 @@ class VenueMechanismLocal(VenueMechanismCore):
             else:
                appScriptName = ""
                appScript     = ""
-               remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+               remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
                self.remoteBatch = RemoteWorkflowPEGASUS(self.hubUserName,self.hubUserId,self.submitterClass,
                                                         self.session,self.instanceToken,self.wsJobId,
                                                         self.runName,self.localJobId,self.instanceId,
@@ -539,7 +541,7 @@ class VenueMechanismLocal(VenueMechanismCore):
                                                         self.isMultiCoreRequest,self.siteInfo,self.submissionScriptsInfo,
                                                         self.managerInfo,self.timeHistoryLogs)
             appScriptName,appScript,appScriptExecutable = self.remoteAppScript.buildAppScript()
-            remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+            remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
             self.remoteBatch = RemoteBatchPEGASUS(self.hubUserName,self.hubUserId,self.submitterClass,
                                                   self.session,self.instanceToken,self.wsJobId,
                                                   self.runName,self.localJobId,self.instanceId,
@@ -573,14 +575,31 @@ class VenueMechanismLocal(VenueMechanismCore):
                   self._writeToStderr(message + "\n")
                   if not exitCode:
                      exitCode = 10
-      elif self.remoteBatchSystem == 'TAPIS':
+      elif self.remoteBatchSystem == 'TAPIS2':
          appScriptName       = ""
          appScript           = ""
          appScriptExecutable = False
-         self.remoteBatch = RemoteBatchTAPIS(self.session,self.instanceToken,self.wsJobId,
-                                             self.localJobId,self.instanceId,
-                                             self.tapisSiteInfo,self.submissionScriptsInfo,
-                                             self.nNodes,self.ppn,self.wallTime)
+         self.remoteBatch = RemoteBatchTAPIS2(self.session,self.instanceToken,self.wsJobId,
+                                              self.runName,self.localJobId,self.instanceId,
+                                              self.tapis2SiteInfo,self.submissionScriptsInfo,
+                                              self.nNodes,self.ppn,self.wallTime)
+         batchScriptName,batchScript = self.remoteBatch.getBatchTapisFile()
+         batchScriptExecutable       = False
+         batchLogName                = ""
+         nodeFileName                = ""
+         nodeList                    = []
+         toolInputTemplateFileName   = ""
+         toolInputTemplate           = ""
+         toolOutputTemplateFileName  = ""
+         toolOutputTemplate          = ""
+      elif self.remoteBatchSystem == 'TAPIS3':
+         appScriptName       = ""
+         appScript           = ""
+         appScriptExecutable = False
+         self.remoteBatch = RemoteBatchTAPIS3(self.session,self.instanceToken,self.wsJobId,
+                                              self.runName,self.localJobId,self.instanceId,
+                                              self.tapis3SiteInfo,self.submissionScriptsInfo,
+                                              self.nNodes,self.ppn,self.wallTime)
          batchScriptName,batchScript = self.remoteBatch.getBatchTapisFile()
          batchScriptExecutable       = False
          batchLogName                = ""
@@ -769,7 +788,7 @@ class VenueMechanismLocal(VenueMechanismCore):
       exitStatus = 0
       self.jobStatistics[self.jobIndex]['venue'] = self.venue
 
-      remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+      remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
       command = self.receiveInput + " " + self.instanceDirectory + " " + \
                                           remoteScratchDirectory + " " + \
                                           os.path.join(self.instanceDirectory,self.timeHistoryLogs['timestampTransferred'])
@@ -833,7 +852,7 @@ class VenueMechanismLocal(VenueMechanismCore):
          self.jobStatistics[self.jobIndex]['exitCode'] = exitStatus
       else:
          commandArgs = []
-         remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+         remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
          commandArgs.append(self.submitBatchJob)
          commandArgs.append(self.instanceDirectory)
          commandArgs.append(remoteScratchDirectory)
@@ -953,10 +972,10 @@ class VenueMechanismLocal(VenueMechanismCore):
             if self.jobSubmitted:
                if (self.isParametric and self.instanceId == "0") or not self.isParametric:
                   if not self.sharedUserSpace:
-                     remoteWorkingDirectory = self.workingDirectory.replace('$','\$')
+                     remoteWorkingDirectory = self.workingDirectory.replace('$',r'\$')
                   else:
                      remoteWorkingDirectory = self.instanceDirectory
-                  remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+                  remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
                   command = self.postProcessJob + " " + remoteWorkingDirectory + " " + \
                                                         remoteScratchDirectory + " " + \
                                                         "PEGASUS"
@@ -1086,7 +1105,7 @@ class VenueMechanismLocal(VenueMechanismCore):
          if   self.remoteBatchSystem == 'PEGASUS':
             if self.jobSubmitted:
                if (self.isParametric and self.instanceId == "0") or not self.isParametric:
-                  remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+                  remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
                   if self.isParametric:
                      self.jobOutput.getPegasusStdTimeFiles(self.instanceDirectory,
                                                            remoteScratchDirectory,
@@ -1119,7 +1138,7 @@ class VenueMechanismLocal(VenueMechanismCore):
                   if os.path.exists(batchTransferredTimePath):
                      shutil.copy2(batchTransferredTimePath,os.path.join(self.instanceDirectory,timestampTransferredFile))
 
-               remoteScratchDirectory = self.scratchDirectory.replace('$','\$')
+               remoteScratchDirectory = self.scratchDirectory.replace('$',r'\$')
                command = self.cleanupJob + " " + self.instanceDirectory + " " + \
                                                  remoteScratchDirectory + " " + \
                                                  self.remoteBatchSystem
